'''
Created on Feb 26, 2016

@author: alevin
'''
from __future__ import absolute_import

import unittest
import pythesint as pti


class GCMDVocabularyTest(unittest.TestCase):

    def test_read_revision(self):
        gcmd_list = []
        line = ('"Keyword Version: 8.1","Revision: 2016-01-08 13:40:40","'
                'Timestamp: 2016-02-26 04:46:02","Terms Of Use: See '
                'http://gcmd.nasa.gov/r/l/TermsOfUse","The most up to date XML'
                ' representations can be found here: '
                'http://gcmdservices.gsfc.nasa.gov/kms/concepts/concept_scheme'
                '/instruments/?format=xml"')
        pti.gcmd_vocabulary._read_revision(line, gcmd_list)
        self.assertEqual(len(gcmd_list), 1)
        self.assertDictEqual(gcmd_list[0], {'Revision': '2016-01-08 13:40:40'})

    def test_check_categories(self):
        line = ('Category,Class,Type,Subtype,Short_Name,Long_Name,UUID')
        categories = ['Category', 'Class', 'Type', 'Subtype', 'Short_Name',
                      'Long_Name']
        pti.gcmd_vocabulary._check_categories(line, categories)

    def test_check_categories_wrong(self):
        line = ('Category,Class,Type,Subtype,Short_Name,Long_Name,UUID')
        categories = ['Category', 'Type', 'Class', 'Subtype', 'Short_Name',
                      'Long_Name']
        with self.assertRaises(TypeError):
            pti.gcmd_vocabulary._check_categories(line, categories)

    def test_read_line_simple(self):
        gcmd_list = [{'Revision': '2016-01-08 13:40:40'}]
        line = ('"Earth Remote Sensing Instruments","","","","","",'
                '"6015ef7b-f3bd-49e1-9193-cc23db566b69"')
        categories = ['Category', 'Class', 'Type', 'Subtype', 'Short_Name',
                      'Long_Name']
        pti.gcmd_vocabulary._read_line(line, gcmd_list, categories)
        self.assertEqual(len(gcmd_list), 2)
        self.assertEqual(gcmd_list[1], {'Category':
                                        'Earth Remote Sensing Instruments',
                                        'Class': '',
                                        'Type': '',
                                        'Subtype': '',
                                        'Short_Name': '',
                                        'Long_Name': ''})

    def test_read_line_full(self):
        gcmd_list = [{'Revision': '2016-01-08 13:40:40'}]
        line = ('"Earth Remote Sensing Instruments","Active Remote Sensing",'
                '"Altimeters","Lidar/Laser Altimeters","GLAS","Geoscience '
                'Laser Altimeter System",'
                '"57463f12-2a21-49f9-9477-718030d34291"')
        categories = ['Category', 'Class', 'Type', 'Subtype', 'Short_Name',
                      'Long_Name']
        pti.gcmd_vocabulary._read_line(line, gcmd_list, categories)
        self.assertEqual(len(gcmd_list), 2)
        self.assertEqual(gcmd_list[1], {'Category':
                                        'Earth Remote Sensing Instruments',
                                        'Class': 'Active Remote Sensing',
                                        'Type': 'Altimeters',
                                        'Subtype': 'Lidar/Laser Altimeters',
                                        'Short_Name': 'GLAS',
                                        'Long_Name':
                                        'Geoscience Laser Altimeter System'})

    def test_read_line_not_applicable(self):
        gcmd_list = [{'Revision': '2016-01-08 13:40:40'}]
        line = ('"NOT APPLICABLE","","","","","",'
                '"8129a4b9-b5f9-4585-87e6-4576c3a53682"')
        categories = ['Category', 'Class', 'Type', 'Subtype', 'Short_Name',
                      'Long_Name']
        pti.gcmd_vocabulary._read_line(line, gcmd_list, categories)
        self.assertEqual(len(gcmd_list), 1)
        self.assertEqual(gcmd_list[0], {'Revision': '2016-01-08 13:40:40'})

    def test_read_line_wrong_length(self):
        gcmd_list = [{'Revision': '2016-01-08 13:40:40'}]
        line = ('"Earth Remote Sensing Instruments","Active Remote Sensing",'
                '"Altimeters","Lidar/Laser Altimeters","GLAS","Geoscience '
                'Laser Altimeter System","Sneaky Extra Element",'
                '"57463f12-2a21-49f9-9477-718030d34291"')
        categories = ['Category', 'Class', 'Type', 'Subtype', 'Short_Name',
                      'Long_Name']
        pti.gcmd_vocabulary._read_line(line, gcmd_list, categories)
        self.assertEqual(len(gcmd_list), 1)
        self.assertEqual(gcmd_list[0], {'Revision': '2016-01-08 13:40:40'})

    def test_get_location_by_type(self):
        type = 'africa'
        a = pti.get_gcmd_location(type)
        self.assertEqual(a['Location_Type'], 'AFRICA')

if __name__ == "__main__":
    # import sys;sys.argv = ['', 'Test.testName']
    unittest.main()



from __future__ import absolute_import
from __future__ import print_function

import requests
import collections
import unittest
import os
import shutil
from pkg_resources import resource_filename, resource_string

import pythesint as pti
from mock.mock import MagicMock, patch
from pythesint.pathsolver import DATA_HOME


class PythesintTest(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        # Delete stored json data to work with a clean slate
        shutil.rmtree(os.path.join(DATA_HOME, 'pythesint', 'json'))

    def test_update_all(self):
        pti.update_all_vocabularies()
        for _, vocab in pti.pythesint.vocabularies.items():
            self.assertIsInstance(vocab.get_list(), list)

    def test_get_list(self):
        # TODO - these should be generated from rc file!
        dicts = ['wkv_variable', 'gcmd_instrument', 'gcmd_science_keyword',
                 'gcmd_provider', 'gcmd_platform', 'gcmd_location',
                 'gcmd_horizontalresolutionrange',
                 'gcmd_verticalresolutionrange',
                 'gcmd_temporalresolutionrange', 'gcmd_project',
                 'gcmd_rucontenttype', 'cf_standard_name',
                 'iso19115_topic_category']
        for name in dicts:
            function = getattr(pti, 'get_' + name + '_list')
            self.assertIsInstance(function(), list)

    def test_remove_and_get_gcmd_instrument(self):
        if os.path.exists(resource_filename('pythesint', 'json')):
            shutil.rmtree(resource_filename('pythesint', 'json'))
        item = 'MERIS'
        self.assertIsInstance(pti.get_gcmd_instrument(item),
                collections.OrderedDict)

    def test_get_gcmd_instrument(self):
        item = 'MERIS'
        self.assertIsInstance(pti.get_gcmd_instrument(item),
                collections.OrderedDict)

    def test_search_gcmd_instrument_list(self):
        item = 'MERIS'
        self.assertIsInstance(pti.search_gcmd_instrument_list(item)[0],
                collections.OrderedDict)

    def test_get_gcmd_science_keyword(self):
        item = 'sigma naught'
        self.assertIsInstance(pti.get_gcmd_science_keyword(item),
                collections.OrderedDict)

    def test_get_gcmd_provider(self):
        item = 'NERSC'
        self.assertIsInstance(pti.get_gcmd_provider(item),
                collections.OrderedDict)

    def test_get_gcmd_platform(self):
        item = 'AQUA'
        self.assertIsInstance(pti.get_gcmd_platform(item),
                collections.OrderedDict)

    def test_get_gcmd_location(self):
        item = 'NORWEGIAN SEA'
        self.assertIsInstance(pti.get_gcmd_location(item),
                collections.OrderedDict)

    def test_get_gcmd_horizontalresolutionrange(self):
        item = '< 1 meter'
        self.assertIsInstance(pti.get_gcmd_horizontalresolutionrange(item),
                collections.OrderedDict)

    def test_get_gcmd_verticalresolutionrange(self):
        item = '< 1 meter'
        self.assertIsInstance(pti.get_gcmd_verticalresolutionrange(item),
                collections.OrderedDict)

    def test_get_gcmd_temporalresolutionrange(self):
        item = 'Decadal'
        self.assertIsInstance(pti.get_gcmd_temporalresolutionrange(item),
                collections.OrderedDict)

    def test_get_gcmd_project(self):
        item = 'IPCC'
        self.assertIsInstance(pti.get_gcmd_project(item),
                collections.OrderedDict)

    def test_get_gcmd_rucontenttype(self):
        item = 'THREDDS DATA'
        self.assertIsInstance(pti.get_gcmd_rucontenttype(item),
                collections.OrderedDict)

    def test_get_cf_standard_name(self):
        item = 'surface_backwards_scattering_coefficient_of_radar_wave'
        self.assertIsInstance(pti.get_cf_standard_name(item),
                collections.OrderedDict)

    def test_get_wkv_variable(self):
        item = 'surface_backwards_doppler_frequency_shift_of_radar_wave_due_to_surface_velocity'
        self.assertIsInstance(pti.get_wkv_variable(item),
                collections.OrderedDict)

    def test_get_wkv_latitude(self):
        item = 'latitude'
        self.assertIsInstance(pti.get_wkv_variable(item),
                collections.OrderedDict)

    def test_get_iso19115_topic_category(self):
        item = 'Oceans'
        self.assertIsInstance(pti.get_iso19115_topic_category(item),
                collections.OrderedDict)

    def test_get_fake_instrument(self):
        item = 'FakeItem'
        self.assertRaises(IndexError, pti.get_gcmd_instrument, item)

    def test_update_functions_exists(self):
        # TODO - these should be generated from rc file!
        functions = ['update_wkv_variable', 'update_gcmd_instrument',
                     'update_gcmd_science_keyword', 'update_gcmd_provider',
                     'update_gcmd_platform', 'update_gcmd_location',
                     'update_gcmd_horizontalresolutionrange',
                     'update_gcmd_verticalresolutionrange',
                     'update_gcmd_temporalresolutionrange',
                     'update_gcmd_project', 'update_gcmd_rucontenttype',
                     'update_cf_standard_name',
                     'update_iso19115_topic_category']
        for function in functions:
            self.assertTrue(hasattr(pti, function), 'Function is missing:%s' %
                            (function))

    def urls(self):
        # items() in general is inefficient on Python 2, but should be no problem with just a couple items.
        for key, voc in pti.pythesint.vocabularies.items():
            if hasattr(voc, 'url'):
                response = requests.get(voc.url)

    def test_update_all_mocked(self):
        orig_vocab = pti.pythesint.vocabularies
        pti.pythesint.vocabularies = {'1': MagicMock(),
                                      'anothervocab': MagicMock(),
                                      'thirdvocab': MagicMock(),
                                      'instruments': MagicMock(),
                                      'something': MagicMock()}
        pti.update_all_vocabularies()
        # items() in general is inefficient on Python 2, but should be no problem with just a couple items.
        for _, mock in pti.pythesint.vocabularies.items():
            mock.update.assert_called_once_with()
        pti.pythesint.vocabularies = orig_vocab

if __name__ == "__main__":
    unittest.main()



from __future__ import absolute_import

from collections import OrderedDict
import requests

from pythesint.json_vocabulary import JSONVocabulary


class GCMDVocabulary(JSONVocabulary):
    def _fetch_online_data(self):
        ''' Return list of GCMD standard keywords
            self.url and self.categories must be set
        '''
        r = requests.get(self.url)
        rlines = [line for line in r.text.splitlines()]
        gcmd_list = []
        _read_revision(rlines[0], gcmd_list)
        _check_categories(rlines[1], self.categories)
        for line in rlines[2:]:
            _read_line(line, gcmd_list, self.categories)

        return gcmd_list


def _read_revision(line, gcmd_list):
    ''' Reads the line, extracts the Revision into a new dictionary and appends
    it to gcmd_list
    '''
    # TODO: Cast exception if not found?
    if 'Keyword Version' and 'Revision' in line:
        meta = line.split('","')
        gcmd_list.append({'Revision': meta[1][10:]})


def _check_categories(line, categories):
    ''' Throws an exception if the line does not match categories
    '''
    kw_groups = line.split(',')
    kw_groups.pop(-1)
    # Make sure the group items are as expected
    if kw_groups != categories:
        raise TypeError('%s is not equal to %s' % (kw_groups, categories))


def _read_line(line, gcmd_list, categories):
    ''' Converts line into dictionary values for elements in the categories
    appends the dictionary to gcmd_list
    '''
    gcmd_keywords = line.split('","')
    gcmd_keywords[0] = gcmd_keywords[0].strip('"')
    if gcmd_keywords[0] == 'NOT APPLICABLE':
        return
    # Remove last item (the ID is not needed)
    gcmd_keywords.pop(-1)
    if len(gcmd_keywords) != len(categories):
        return
    line_kw = OrderedDict()
    for i, key in enumerate(categories):
        line_kw[key] = gcmd_keywords[i]
    gcmd_list.append(line_kw)



from __future__ import absolute_import

import yaml
import requests

from pythesint.json_vocabulary import JSONVocabulary


class WKVVocabulary(JSONVocabulary):
    def _fetch_online_data(self):
        ''' Return list of Well Known Variables from Nansat        '''
        r = requests.get(self.url)
        return yaml.load(r.text)

